(function (root, factory) {
  if (typeof exports === 'object') {
    module.exports = factory();
  } else {
    root.logger = factory();
  }
}(this, function () {
  function createMessage(level, text, extra) {
    const loggedAt = new Date();
  
    return {
      loggedAt: loggedAt,
      level: level,
      text: text,
      extra: extra || {}
    };
  }
  
  function createLogger(label) {
    if (typeof label !== 'string') {
      throw new TypeError('Label is required');
    }
  
    var endpoint = 'http://localhost:9000/logger';
    var isFetching = false;
    var timeout = 5000;
    var timeoutId;
    
    var queue = [];
    try {
      queue = JSON.parse(localStorage.getItem('logger:' + label)) || [];
    } catch (e) {}
  
    window.onbeforeunload = function () {
      try {
        localStorage.setItem('logger:' + label, JSON.stringify(queue));
      } catch (e) {}
    };

    function setEndpoint(newURL) {
      endpoint = newURL;
    }
  
    function handleRequestFail() {
      if (timeoutId) {
        clearTimeout(timeoutId);
      }
  
      isFetching = false;
      timeoutId = setTimeout(handleQueueChange, timeout);
    }
  
    function handleQueueChange() {
      if (timeoutId) {
        clearTimeout(timeoutId);
      }
  
      if (!isFetching && queue.length > 0) {
        isFetching = true;
  
        var firstMessage = Object.assign({}, queue[0], {
          sentAt: new Date()
        });
  
        var xhr = new XMLHttpRequest();
        xhr.open('POST', endpoint);
        xhr.setRequestHeader('Content-Type', 'application/json');
        
        xhr.onload = function () {
          if (!(this.status >= 200 && this.status < 300)) {
            return handleRequestFail();
          }
          isFetching = false;
          queue.shift();
          handleQueueChange();
        };
  
        xhr.onerror = function () {
          handleRequestFail();
        };
  
        xhr.send(JSON.stringify(firstMessage));
      }
    }
  
    function append(message) {
      queue.push(message);
      handleQueueChange();
    }
  
    function info(text, extra) {
      append(createMessage('INFO', text, extra));
    }
  
    function warn(text, extra) {
      append(createMessage('WARN', text, extra));
    }
  
    function error(text, extra) {
      append(createMessage('ERROR', text, extra));
    }
  
    return {
      setEndpoint: setEndpoint,
      info: info,
      warn: warn,
      error: error
    };
  }

  return {
    createLogger: createLogger
  };
}));
